@@ -1,13 +1,10 @@
 import sys
-from neocore.IO.Mixins import SerializableMixin
+from neo.Core.IO.Mixins import SerializableMixin
 from neo.Core.Size import Size as s
 from neo.Core.Size import GetVarSize
 
 
 class CoinReference(SerializableMixin):
-    PrevHash = None
-
-    PrevIndex = None
 
     def __init__(self, prev_hash=None, prev_index=None):
         """
@@ -61,16 +58,14 @@ class CoinReference(SerializableMixin):
         """
         if other is None:
             return False
+        if type(other) is not CoinReference:
+            return False
         if other.PrevHash.ToBytes() == self.PrevHash.ToBytes() and other.PrevIndex == self.PrevIndex:
             return True
         return False
 
     def __eq__(self, other):
-        if other is None:
-            return False
-        if other.PrevHash.ToBytes() == self.PrevHash.ToBytes() and other.PrevIndex == self.PrevIndex:
-            return True
-        return False
+        return self.Equals(other)
 
     def __hash__(self):
         return int.from_bytes(self.PrevHash.Data + bytearray(self.PrevIndex), 'little')
