@@ -1,19 +1,10 @@
-from neocore.IO.Mixins import SerializableMixin
-from neocore.Cryptography.Crypto import Crypto
-from neocore.BigInteger import BigInteger
+from neo.Core.IO.Mixins import SerializableMixin
+from neo.Core.Cryptography.Crypto import Crypto
+from neo.Core.BigInteger import BigInteger
 from neo.SmartContract.ContractParameterType import ContractParameterType, ToName
 
 
 class FunctionCode(SerializableMixin):
-    Script = bytearray()
-
-    ParameterList = bytearray()
-
-    ReturnType = None
-
-    _scriptHash = None
-
-    ContractProperties = None
 
     @property
     def ReturnTypeBigInteger(self):
@@ -53,6 +44,7 @@ class FunctionCode(SerializableMixin):
         return self.ContractProperties & ContractPropertyState.Payable > 0
 
     def __init__(self, script=None, param_list=None, return_type=255, contract_properties=0):
+        self._scriptHash = None
         self.Script = script
         if param_list is None:
             self.ParameterList = []
@@ -84,7 +76,7 @@ class FunctionCode(SerializableMixin):
         """
         self.Script = reader.ReadVarBytes()
         self.ParameterList = reader.ReadVarBytes()
-        self.ReturnType = reader.ReadByte()
+        self.ReturnType = ord(reader.ReadByte())
 
     def Serialize(self, writer):
         """
