@@ -1,21 +1,16 @@
 import sys
 from .StateBase import StateBase
-from neocore.Fixed8 import Fixed8
-from neocore.IO.BinaryReader import BinaryReader
+from neo.Core.Fixed8 import Fixed8
+from neo.Core.IO.BinaryReader import BinaryReader
 from neo.IO.MemoryStream import StreamManager
-from neocore.Cryptography.Crypto import Crypto
-from neocore.IO.BinaryWriter import BinaryWriter
+from neo.Core.Cryptography.Crypto import Crypto
+from neo.Core.IO.BinaryWriter import BinaryWriter
 
 from neo.Core.Size import Size as s
 from neo.Core.Size import GetVarSize
 
 
 class AccountState(StateBase):
-    ScriptHash = None
-    IsFrozen = False
-    Votes = []
-    Balances = {}
-
     def __init__(self, script_hash=None, is_frozen=False, votes=None, balances=None):
         """
         Create an instance.
@@ -114,7 +109,7 @@ class AccountState(StateBase):
         Deserialize full object.
 
         Args:
-            reader (neocore.IO.BinaryReader):
+            reader (neo.Core.IO.BinaryReader):
         """
         super(AccountState, self).Deserialize(reader)
         self.ScriptHash = reader.ReadUInt160()
