@@ -8,29 +8,33 @@ import shutil
 import os
 
 
-def BootstrapBlockchainFile(target_dir, download_location, bootstrap_name, require_confirm=True):
+def BootstrapBlockchainFile(target_dir, download_location, bootstrap_name, require_confirm=True, delete_bootstrap_file=True):
     if download_location is None:
         print("no bootstrap location file specified. Please update your configuration file.")
         sys.exit(0)
 
     if require_confirm:
         print("This will overwrite any data currently in %s.\nType 'confirm' to continue" % target_dir)
-        confirm = prompt("[confirm]> ", is_password=False)
+        try:
+            confirm = prompt("[confirm]> ", is_password=False)
+        except KeyboardInterrupt:
+            confirm = False
         if confirm == 'confirm':
-            return do_bootstrap(download_location, bootstrap_name, target_dir)
+            return do_bootstrap(download_location, bootstrap_name, target_dir, delete_bootstrap_file=delete_bootstrap_file)
     else:
 
         return do_bootstrap(download_location,
                             bootstrap_name,
                             target_dir,
                             tmp_file_name=os.path.join(settings.DATA_DIR_PATH, 'btest.tar.gz'),
-                            tmp_chain_name='btestchain')
+                            tmp_chain_name='btestchain',
+                            delete_bootstrap_file=delete_bootstrap_file)
 
     print("bootstrap cancelled")
     sys.exit(0)
 
 
-def do_bootstrap(download_location, bootstrap_name, destination_dir, tmp_file_name=None, tmp_chain_name='tmpchain'):
+def do_bootstrap(download_location, bootstrap_name, destination_dir, tmp_file_name=None, tmp_chain_name='tmpchain', delete_bootstrap_file=True):
     if tmp_file_name is None:
         tmp_file_name = os.path.join(settings.DATA_DIR_PATH, 'bootstrap.tar.gz')
 
@@ -97,6 +101,10 @@ def do_bootstrap(download_location, bootstrap_name, destination_dir, tmp_file_na
         if os.path.exists(tmp_chain_name):
             shutil.rmtree(tmp_chain_name)
 
+        if delete_bootstrap_file and os.path.exists(tmp_file_name):
+            print("removing temp bootstrap file %s " % tmp_file_name)
+            os.remove(tmp_file_name)
+
     if success:
         print("Successfully downloaded bootstrap chain!")
 
