@@ -1,12 +1,13 @@
-from prompt_toolkit import prompt
+from neo.Network.common import blocking_prompt as prompt
 from neo.logging import log_manager
 from neo.Prompt.CommandBase import CommandBase, CommandDesc, ParameterDesc
 from neo.Prompt.Utils import get_arg
 from neo.Settings import settings
-from neo.Network.NodeLeader import NodeLeader
 from neo.Prompt.PromptPrinter import prompt_print as print
 from distutils import util
+from neo.Network.nodemanager import NodeManager
 import logging
+from neo.Network.common import wait_for
 
 
 class CommandConfig(CommandBase):
@@ -17,8 +18,8 @@ class CommandConfig(CommandBase):
         self.register_sub_command(CommandConfigSCEvents())
         self.register_sub_command(CommandConfigDebugNotify())
         self.register_sub_command(CommandConfigVMLog())
-        self.register_sub_command(CommandConfigNodeRequests())
         self.register_sub_command(CommandConfigMaxpeers())
+        self.register_sub_command(CommandConfigMinpeers())
         self.register_sub_command(CommandConfigNEP8())
 
     def command_desc(self):
@@ -43,7 +44,12 @@ class CommandConfigOutput(CommandBase):
         super().__init__()
 
     def execute(self, arguments=None):
-        return start_output_config()
+        try:
+            res = start_output_config()
+        except KeyboardInterrupt:
+            print('Output configuration cancelled')
+            return
+        return res
 
     def command_desc(self):
         return CommandDesc('output', 'configure the log output level settings')
@@ -133,35 +139,6 @@ class CommandConfigVMLog(CommandBase):
         return CommandDesc('vm-log', 'toggle VM instruction execution logging to file', [p1])
 
 
-class CommandConfigNodeRequests(CommandBase):
-    def __init__(self):
-        super().__init__()
-
-    def execute(self, arguments):
-        if len(arguments) in [1, 2]:
-            if len(arguments) == 2:
-                try:
-                    return NodeLeader.Instance().setBlockReqSizeAndMax(int(arguments[0]), int(arguments[1]))
-                except ValueError:
-                    print("Invalid values. Please specify a block request part and max size for each node, like 30 and 1000")
-                    return False
-            elif len(arguments) == 1:
-                return NodeLeader.Instance().setBlockReqSizeByName(arguments[0])
-        else:
-            print("Please specify the required parameter")
-            return False
-
-    def command_desc(self):
-        p1 = ParameterDesc('block-size', 'preset of "slow"/"normal"/"fast", or a specific block request size (max. 500) e.g. 250 ')
-        p2 = ParameterDesc('queue-size', 'maximum number of outstanding block requests')
-        return CommandDesc('node-requests', 'configure block request settings', [p1, p2])
-
-    def handle_help(self, arguments):
-        super().handle_help(arguments)
-        print(f"\nCurrent settings {self.command_desc().params[0].name}:"
-              f" {NodeLeader.Instance().BREQPART} {self.command_desc().params[1].name}: {NodeLeader.Instance().BREQMAX}")
-
-
 class CommandConfigNEP8(CommandBase):
     def __init__(self):
         super().__init__()
@@ -197,24 +174,39 @@ class CommandConfigMaxpeers(CommandBase):
     def execute(self, arguments):
         c1 = get_arg(arguments)
         if c1 is not None:
-            try:
-                current_max = settings.CONNECTED_PEER_MAX
-                settings.set_max_peers(c1)
-                c1 = int(c1)
-                p_len = len(NodeLeader.Instance().Peers)
-                if c1 < current_max and c1 < p_len:
-                    to_remove = p_len - c1
-                    peers = NodeLeader.Instance().Peers
-                    for i in range(to_remove):
-                        peer = peers[-1]  # disconnect last peer added first
-                        peer.Disconnect("Max connected peers reached", isDead=False)
-                        peers.pop()
 
-                print(f"Maxpeers set to {c1}")
-                return c1
+            try:
+                c1 = int(c1)
+            except ValueError:
+                print("Invalid argument")
+                return
+
+            if c1 > 10:
+                print("Max peers is limited to 10")
+                return
+
+            try:
+                settings.set_max_peers(c1)
+                if c1 < settings.CONNECTED_PEER_MIN:
+                    settings.set_min_peers(c1)
+                    print(f"Minpeers set to {c1}")
             except ValueError:
                 print("Please supply a positive integer for maxpeers")
                 return
+
+            nodemgr = NodeManager()
+            nodemgr.max_clients = c1
+
+            current_max = settings.CONNECTED_PEER_MAX
+            connected_count = len(nodemgr.nodes)
+            if current_max < connected_count:
+                to_remove = connected_count - c1
+                for _ in range(to_remove):
+                    last_connected_node = nodemgr.nodes[-1]
+                    wait_for(last_connected_node.disconnect())  # need to avoid it being labelled as dead/bad
+
+            print(f"Maxpeers set to {c1}")
+            return c1
         else:
             print(f"Maintaining maxpeers at {settings.CONNECTED_PEER_MAX}")
             return
@@ -224,6 +216,33 @@ class CommandConfigMaxpeers(CommandBase):
         return CommandDesc('maxpeers', 'configure number of max peers', [p1])
 
 
+class CommandConfigMinpeers(CommandBase):
+    def __init__(self):
+        super().__init__()
+
+    def execute(self, arguments):
+        c1 = get_arg(arguments)
+        if c1 is not None:
+            try:
+                c1 = int(c1)
+                if c1 > settings.CONNECTED_PEER_MAX:
+                    print('minpeers setting cannot be bigger than maxpeers setting')
+                    return
+                settings.set_min_peers(c1)
+            except ValueError:
+                print("Please supply a positive integer for minpeers")
+                return
+            print(f"Minpeers set to {c1}")
+            return c1
+        else:
+            print(f"Maintaining minpeers at {settings.CONNECTED_PEER_MIN}")
+            return
+
+    def command_desc(self):
+        p1 = ParameterDesc('number', 'minimum number of nodes to connect to')
+        return CommandDesc('minpeers', 'configure number of min peers', [p1])
+
+
 def start_output_config():
     # temporarily mute stdout while we try to reconfigure our settings
     # components like `network` set at DEBUG level will spam through the console
