@@ -3,15 +3,15 @@ from neo.Core.TX.ClaimTransaction import ClaimTransaction
 from neo.Core.TX.Transaction import TransactionOutput
 from neo.Core.TX.TransactionAttribute import TransactionAttribute, TransactionAttributeUsage
 from neo.SmartContract.ContractParameterContext import ContractParametersContext
-from neo.Network.NodeLeader import NodeLeader
 from neo.Prompt import Utils as PromptUtils
 from neo.Wallets.utils import to_aes_key
 from neo.Implementations.Wallets.peewee.UserWallet import UserWallet
-from neocore.Fixed8 import Fixed8
-from neocore.UInt160 import UInt160
-from prompt_toolkit import prompt
+from neo.Core.Fixed8 import Fixed8
+from neo.Core.UInt160 import UInt160
+from neo.Network.common import blocking_prompt as prompt
 import json
 import os
+import asyncio
 from neo.Prompt.CommandBase import CommandBase, CommandDesc, ParameterDesc
 from neo.Prompt.PromptData import PromptData
 from neo.Prompt.Commands.Send import CommandWalletSend, CommandWalletSendMany, CommandWalletSign
@@ -20,8 +20,9 @@ from neo.Prompt.Commands.WalletAddress import CommandWalletAddress
 from neo.Prompt.Commands.WalletImport import CommandWalletImport
 from neo.Prompt.Commands.WalletExport import CommandWalletExport
 from neo.logging import log_manager
-from neocore.Utils import isValidPublicAddress
+from neo.Core.Utils import isValidPublicAddress
 from neo.Prompt.PromptPrinter import prompt_print as print
+from neo.Network.nodemanager import NodeManager
 
 logger = log_manager.getLogger()
 
@@ -61,7 +62,7 @@ class CommandWallet(CommandBase):
             return
 
         if not item:
-            print("Wallet %s " % json.dumps(wallet.ToJson(), indent=4))
+            wallet.pretty_print(item)
             return wallet
 
         try:
@@ -94,8 +95,12 @@ class CommandWalletCreate(CommandBase):
         if PromptData.Wallet:
             PromptData.close_wallet()
 
-        passwd1 = prompt("[password]> ", is_password=True)
-        passwd2 = prompt("[password again]> ", is_password=True)
+        try:
+            passwd1 = prompt("[password]> ", is_password=True)
+            passwd2 = prompt("[password again]> ", is_password=True)
+        except KeyboardInterrupt:
+            print("Wallet creation cancelled")
+            return
 
         if passwd1 != passwd2 or len(passwd1) < 10:
             print("Please provide matching passwords that are at least 10 characters long")
@@ -120,7 +125,7 @@ class CommandWalletCreate(CommandBase):
             return
 
         if PromptData.Wallet:
-            PromptData.Prompt.start_wallet_loop()
+            asyncio.create_task(PromptData.Wallet.sync_wallet(start_block=PromptData.Wallet._current_height))
             return PromptData.Wallet
 
     def command_desc(self):
@@ -147,14 +152,17 @@ class CommandWalletOpen(CommandBase):
             print("Wallet file not found")
             return
 
-        passwd = prompt("[password]> ", is_password=True)
+        try:
+            passwd = prompt("[password]> ", is_password=True)
+        except KeyboardInterrupt:
+            print("Wallet opening cancelled")
+            return
         password_key = to_aes_key(passwd)
 
         try:
             PromptData.Wallet = UserWallet.Open(path, password_key)
-
-            PromptData.Prompt.start_wallet_loop()
             print("Opened wallet at %s" % path)
+            asyncio.create_task(PromptData.Wallet.sync_wallet(start_block=PromptData.Wallet._current_height))
             return PromptData.Wallet
         except Exception as e:
             print("Could not open wallet: %s" % e)
@@ -182,8 +190,9 @@ class CommandWalletVerbose(CommandBase):
         super().__init__()
 
     def execute(self, arguments=None):
-        print("Wallet %s " % json.dumps(PromptData.Wallet.ToJson(verbose=True), indent=4))
-        return True
+        wallet = PromptData.Wallet
+        wallet.pretty_print(verbose=True)
+        return wallet
 
     def command_desc(self):
         return CommandDesc('verbose', 'show additional wallet details')
@@ -203,11 +212,13 @@ class CommandWalletClaimGas(CommandBase):
             args, from_addr_str = PromptUtils.get_from_addr(args)
             args, to_addr_str = PromptUtils.get_to_addr(args)
 
-        return ClaimGas(PromptData.Wallet, True, from_addr_str, to_addr_str)
+        return ClaimGas(PromptData.Wallet, from_addr_str, to_addr_str)
 
     def command_desc(self):
         p1 = ParameterDesc('--from-addr', 'source address to claim gas from (if not specified, take first address in wallet)', optional=True)
-        p2 = ParameterDesc('--to-addr', 'destination address for claimed gas (if not specified, take first address in wallet; or, use the from address, if specified)', optional=True)
+        p2 = ParameterDesc('--to-addr',
+                           'destination address for claimed gas (if not specified, take first address in wallet; or, use the from address, if specified)',
+                           optional=True)
         return CommandDesc('claim', 'claim gas', params=[p1, p2])
 
 
@@ -217,16 +228,12 @@ class CommandWalletRebuild(CommandBase):
         super().__init__()
 
     def execute(self, arguments):
-        PromptData.Prompt.stop_wallet_loop()
-
         start_block = PromptUtils.get_arg(arguments, 0, convert_to_int=True)
         if not start_block or start_block < 0:
             start_block = 0
         print(f"Restarting at block {start_block}")
-
-        PromptData.Wallet.Rebuild(start_block)
-
-        PromptData.Prompt.start_wallet_loop()
+        task = asyncio.create_task(PromptData.Wallet.sync_wallet(start_block, rebuild=True))
+        return task
 
     def command_desc(self):
         p1 = ParameterDesc('start_block', 'block number to start the resync at', optional=True)
@@ -274,12 +281,27 @@ class CommandWalletUnspent(CommandBase):
 #########################################################################
 #########################################################################
 
+async def sync_wallet(start_block, rebuild=False):
+    Blockchain.Default().PersistCompleted.on_change -= PromptData.Wallet.ProcessNewBlock
 
-def ClaimGas(wallet, require_password=True, from_addr_str=None, to_addr_str=None):
+    if rebuild:
+        PromptData.Wallet.Rebuild(start_block)
+    while True:
+        # trying with 100, might need to lower if processing takes too long
+        PromptData.Wallet.ProcessBlocks(block_limit=100)
+
+        if PromptData.Wallet.IsSynced:
+            break
+        # give some time to other tasks
+        await asyncio.sleep(0.05)
+
+    Blockchain.Default().PersistCompleted.on_change += PromptData.Wallet.ProcessNewBlock
+
+
+def ClaimGas(wallet, from_addr_str=None, to_addr_str=None):
     """
     Args:
         wallet:
-        require_password:
         from_addr_str:
     Returns:
         (claim transaction, relayed status)
@@ -342,14 +364,16 @@ def ClaimGas(wallet, require_password=True, from_addr_str=None, to_addr_str=None
     print(f"Will make claim for {available_bonus.ToString()} GAS")
     print("------------------------------------------------------------------\n")
 
-    if require_password:
-        print("Enter your password to complete this claim")
+    print("Enter your password to complete this claim")
 
+    try:
         passwd = prompt("[Password]> ", is_password=True)
-
-        if not wallet.ValidatePassword(passwd):
-            print("Incorrect password")
-            return None, False
+    except KeyboardInterrupt:
+        print("Claim transaction cancelled")
+        return None, False
+    if not wallet.ValidatePassword(passwd):
+        print("Incorrect password")
+        return None, False
 
     if context.Completed:
 
@@ -357,7 +381,8 @@ def ClaimGas(wallet, require_password=True, from_addr_str=None, to_addr_str=None
 
         print("claim tx: %s " % json.dumps(claim_tx.ToJson(), indent=4))
 
-        relayed = NodeLeader.Instance().Relay(claim_tx)
+        nodemgr = NodeManager()
+        relayed = nodemgr.relay(claim_tx)
 
         if relayed:
             print("Relayed Tx: %s " % claim_tx.Hash.ToString())
