@@ -1,3 +1,6 @@
+from neo.Core.Blockchain import Blockchain
+
+
 class PromptData:
     Prompt = None
     Wallet = None
@@ -8,7 +11,7 @@ class PromptData:
             return False
 
         path = PromptData.Wallet._path
-        PromptData.Prompt.stop_wallet_loop()
+        Blockchain.Default().PersistCompleted.on_change -= PromptData.Wallet.ProcessNewBlock
         PromptData.Wallet.Close()
         PromptData.Wallet = None
         print("Closed wallet %s" % path)
