@@ -4,24 +4,26 @@ from neo.SmartContract.ApplicationEngine import ApplicationEngine
 from neo.VM.ExecutionContext import ExecutionContext
 from neo.SmartContract import TriggerType
 from mock import Mock, MagicMock
+from neo.VM.Script import Script
 
 
 class TestApplicationEngine(NeoTestCase):
 
     def setUp(self):
-        self.engine = ApplicationEngine(TriggerType.Application, Mock(), Mock(), Mock(), MagicMock())
-        self.engine._Crypto = Mock()
+        gas_mock = MagicMock()
+        gas_mock.value.return_value = 0
+        self.engine = ApplicationEngine(TriggerType.Application, Mock(), Mock(), gas_mock, MagicMock())
 
     def test_get_item_count(self):
-        econtext1 = ExecutionContext(engine=self.engine)
+        econtext1 = ExecutionContext(Script(self.engine.Crypto, b''), 0)
         # 4 items in context 1
-        map = Map({'a': 1, 'b': 2, 'c': 3})
+        map = Map.FromDictionary({'a': 1, 'b': 2, 'c': 3})
         my_int = Integer(BigInteger(1))
         econtext1.EvaluationStack.PushT(map)
         econtext1.EvaluationStack.PushT(my_int)
 
         # 3 items in context 2
-        econtext2 = ExecutionContext(engine=self.engine)
+        econtext2 = ExecutionContext(Script(self.engine.Crypto, b''), 0)
         my_array = Array([my_int, my_int])
         econtext2.EvaluationStack.PushT(my_array)
         econtext2.AltStack.PushT(my_int)
