@@ -8,6 +8,7 @@ class VMFault(Enum):
     UNKNOWN2 = auto()
     INVALID_CONTRACT = auto()
     SYSCALL_ERROR = auto()
+    SYSCALL_INSUFFICIENT_DATA = auto()
     UNKNOWN3 = auto()
     UNKNOWN4 = auto()
     UNKNOWN5 = auto()
@@ -46,3 +47,19 @@ class VMFault(Enum):
     UNKNOWN_STACKISOLATION = auto()
     UNKNOWN_STACKISOLATION2 = auto()
     UNKNOWN_STACKISOLATION3 = auto()
+
+    SETITEM_INVALID_MAP = auto()
+    NEWARRAY_EXCEED_ARRAYLIMIT = auto()
+    NEWARRAY_NEGATIVE_COUNT = auto()
+    NEWSTRUCT_EXCEED_ARRAYLIMIT = auto()
+    NEWSTRUCT_NEGATIVE_COUNT = auto()
+    APPEND_EXCEED_ARRAYLIMIT = auto()
+    PUSHDATA_EXCEED_MAXITEMSIZE = auto()
+    CAT_EXCEED_MAXITEMSIZE = auto()
+    CALL_EXCEED_MAX_INVOCATIONSTACK_SIZE = auto()
+    CALL__I_EXCEED_MAX_INVOCATIONSTACK_SIZE = auto()
+    APPCALL_EXCEED_MAX_INVOCATIONSTACK_SIZE = auto()
+    UNKNOWN_EXCEED_MAX_INVOCATIONSTACK_SIZE = auto()
+    BIGINTEGER_EXCEED_LIMIT = auto()
+    INVALID_SHIFT = auto()
+    INVALID_STACKSIZE = auto()
