@@ -2,15 +2,16 @@ import binascii
 import traceback
 from decimal import Decimal
 from neo.Core.VerificationCode import VerificationCode
-from neocore.Cryptography.Crypto import Crypto
-from neocore.Fixed8 import Fixed8
+from neo.Core.Cryptography.Crypto import Crypto
+from neo.Core.Fixed8 import Fixed8
 from neo.Prompt.Commands.Invoke import TestInvokeContract, test_invoke
 from neo.Prompt import Utils as PromptUtils
-from neocore.UInt160 import UInt160
+from neo.Core.UInt160 import UInt160
 from neo.VM.ScriptBuilder import ScriptBuilder
 from neo.SmartContract.ApplicationEngine import ApplicationEngine
 from neo.Core.Mixins import SerializableMixin
 from neo.logging import log_manager
+from neo.Blockchain import GetBlockchain
 
 logger = log_manager.getLogger()
 
@@ -94,10 +95,12 @@ class NEP5Token(VerificationCode, SerializableMixin):
         sb.EmitAppCallWithOperation(self.ScriptHash, 'symbol')
         sb.EmitAppCallWithOperation(self.ScriptHash, 'decimals')
 
+        snapshot = GetBlockchain().Default()._db.createSnapshot().Clone()
         engine = None
         try:
-            engine = ApplicationEngine.Run(sb.ToArray(), exit_on_error=True, gas=Fixed8.FromDecimal(10.0), test_mode=False)
+            engine = ApplicationEngine.Run(snapshot, sb.ToArray(), exit_on_error=True, gas=Fixed8.FromDecimal(10.0), test_mode=False)
         except Exception as e:
+            traceback.print_exc()
             pass
 
         if engine and len(engine.ResultStack.Items) == 3:
@@ -203,7 +206,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
         invoke_args = [self.ScriptHash.ToString(), 'transferFrom',
                        [PromptUtils.parse_param(from_addr, wallet), PromptUtils.parse_param(to_addr, wallet), PromptUtils.parse_param(amount)]]
 
-        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, None, True)
+        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args)
 
         return tx, fee, results
 
@@ -225,7 +228,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
         invoke_args = [self.ScriptHash.ToString(), 'allowance',
                        [PromptUtils.parse_param(owner_addr, wallet), PromptUtils.parse_param(requestor_addr, wallet)]]
 
-        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, None, True)
+        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args)
 
         return tx, fee, results
 
@@ -248,7 +251,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
         invoke_args = [self.ScriptHash.ToString(), 'approve',
                        [PromptUtils.parse_param(owner_addr, wallet), PromptUtils.parse_param(requestor_addr, wallet), PromptUtils.parse_param(amount)]]
 
-        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, None, True)
+        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args)
 
         return tx, fee, results
 
@@ -271,7 +274,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
 
         invoke_args = invoke_args + attachment_args
 
-        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, None, True, from_addr=mint_to_addr, invoke_attrs=invoke_attrs)
+        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, from_addr=mint_to_addr, invoke_attrs=invoke_attrs)
 
         return tx, fee, results
 
@@ -292,7 +295,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
         invoke_args = [self.ScriptHash.ToString(), 'crowdsale_register',
                        [PromptUtils.parse_param(p, wallet) for p in register_addresses]]
 
-        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, None, True, from_addr)
+        tx, fee, results, num_ops, engine_success = TestInvokeContract(wallet, invoke_args, from_addr=from_addr)
 
         return tx, fee, results
 
@@ -300,7 +303,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
         """
         Serialize this token data to bytes
         Args:
-            writer (neocore.IO.BinaryWriter): binary writer to write serialization data to
+            writer (neo.Core.IO.BinaryWriter): binary writer to write serialization data to
 
         """
         writer.WriteVarString(self.name)
@@ -311,7 +314,7 @@ class NEP5Token(VerificationCode, SerializableMixin):
         """
         Read serialized data from byte stream
         Args:
-            reader (neocore.IO.BinaryReader): reader to read byte data from
+            reader (neo.Core.IO.BinaryReader): reader to read byte data from
         """
         self.name = reader.ReadVarString().decode('utf-8')
         self.symbol = reader.ReadVarString().decode('utf-8')
