@@ -6,27 +6,28 @@ Usage:
 """
 import traceback
 import hashlib
+import asyncio
 from itertools import groupby
 from base58 import b58decode
 from decimal import Decimal
 from Crypto import Random
 from Crypto.Cipher import AES
 from threading import RLock
-from neo.Core.TX.Transaction import TransactionType, TransactionOutput, TXFeeError
+from neo.Core.TX.Transaction import TransactionType, TransactionOutput
 from neo.Core.State.CoinState import CoinState
 from neo.Core.Blockchain import Blockchain
 from neo.Core.CoinReference import CoinReference
 from neo.Core.TX.ClaimTransaction import ClaimTransaction
-from neocore.Cryptography.Helper import scripthash_to_address
-from neocore.Cryptography.Crypto import Crypto
+from neo.Core.Cryptography.Helper import scripthash_to_address
+from neo.Core.Cryptography.Crypto import Crypto
 from neo.Wallets.AddressState import AddressState
 from neo.Wallets.Coin import Coin
-from neocore.KeyPair import KeyPair
+from neo.Core.KeyPair import KeyPair
 from neo.Wallets import NEP5Token
 from neo.Settings import settings
-from neocore.Fixed8 import Fixed8
-from neocore.UInt160 import UInt160
-from neocore.UInt256 import UInt256
+from neo.Core.Fixed8 import Fixed8
+from neo.Core.UInt160 import UInt160
+from neo.Core.UInt256 import UInt256
 from neo.Core.Helper import Helper
 from neo.Wallets.utils import to_aes_key
 from neo.logging import log_manager
@@ -35,22 +36,6 @@ logger = log_manager.getLogger()
 
 
 class Wallet:
-    AddressVersion = None
-
-    _path = ''
-    _iv = None
-    _master_key = None
-    _keys = {}  # holds keypairs
-    _contracts = {}  # holds Contracts
-    _tokens = {}  # holds references to NEP5 tokens
-    _watch_only = []  # holds set of hashes
-    _coins = {}  # holds Coin References
-
-    _current_height = 0
-
-    _vin_exclude = None
-
-    _lock = None  # allows locking for threads that may need to access the DB concurrently (e.g. ProcessBlocks and Rebuild)
 
     @property
     def WalletHeight(self):
@@ -66,7 +51,10 @@ class Wallet:
             passwordKey (aes_key): A password that has been converted to aes key with neo.Wallets.utils.to_aes_key
             create (bool): Whether to create the wallet or simply open.
         """
-
+        self._tokens = {}  # holds references to NEP5 tokens
+        self._watch_only = []  # holds set of hashes
+        self._current_height = 0
+        self._vin_exclude = None
         self.AddressVersion = settings.ADDRESS_VERSION
         self._path = path
         self._lock = RLock()
@@ -650,14 +638,14 @@ class Wallet:
         self._lock.acquire()
         try:
             blockcount = 0
-            while self._current_height <= Blockchain.Default().Height and (block_limit == 0 or blockcount < block_limit):
+            while self._current_height < Blockchain.Default().Height and (block_limit == 0 or blockcount < block_limit):
 
                 block = Blockchain.Default().GetBlockByHeight(self._current_height)
 
                 if block is not None:
                     self.ProcessNewBlock(block)
                 else:
-                    self._current_height += 1
+                    break
 
                 blockcount += 1
 
@@ -998,8 +986,7 @@ class Wallet:
                         use_standard=False,
                         watch_only_val=0,
                         exclude_vin=None,
-                        use_vins_for_asset=None,
-                        skip_fee_calc=False):
+                        use_vins_for_asset=None):
         """
         This method is used to to calculate the necessary TransactionInputs (CoinReferences) and TransactionOutputs to
         be used when creating a transaction that involves an exchange of system assets, ( NEO, Gas, etc ).
@@ -1013,10 +1000,9 @@ class Wallet:
             watch_only_val (int): 0 or CoinState.WATCH_ONLY, if present only choose coins that are in a WatchOnly address.
             exclude_vin (list): A list of CoinReferences to NOT use in the making of this tx.
             use_vins_for_asset (list): A list of CoinReferences to use.
-            skip_fee_calc (bool): If true, the network fee calculation and verification will be skipped.
 
         Returns:
-            tx: (Transaction) Returns the transaction with oupdated inputs and outputs.
+            tx: (Transaction) Returns the transaction with updated inputs and outputs.
         """
 
         tx.ResetReferences()
@@ -1105,14 +1091,6 @@ class Wallet:
         tx.inputs = inputs
         tx.outputs = tx.outputs + new_outputs
 
-        # calculate and verify the required network fee for the tx
-        if tx.Size() > settings.MAX_FREE_TX_SIZE and not skip_fee_calc:
-            req_fee = Fixed8.FromDecimal(settings.FEE_PER_EXTRA_BYTE * (tx.Size() - settings.MAX_FREE_TX_SIZE))
-            if req_fee < settings.LOW_PRIORITY_THRESHOLD:
-                req_fee = settings.LOW_PRIORITY_THRESHOLD
-            if fee < req_fee:
-                raise TXFeeError(f'Transaction cancelled. The tx size ({tx.Size()}) exceeds the max free tx size ({settings.MAX_FREE_TX_SIZE}).\nA network fee of {req_fee.ToString()} GAS is required.')
-
         return tx
 
     def SaveTransaction(self, tx):
@@ -1267,6 +1245,25 @@ class Wallet:
         else:
             return True
 
+    def pretty_print(self, verbose=False):
+        pass
+
+    async def sync_wallet(self, start_block, rebuild=False):
+        Blockchain.Default().PersistCompleted.on_change -= self.ProcessNewBlock
+
+        if rebuild:
+            self.Rebuild(start_block)
+        while True:
+            # trying with 100, might need to lower if processing takes too long
+            self.ProcessBlocks(block_limit=100)
+
+            if self.IsSynced:
+                break
+            # give some time to other tasks
+            await asyncio.sleep(0.05)
+
+        Blockchain.Default().PersistCompleted.on_change += self.ProcessNewBlock
+
     def ToJson(self, verbose=False):
         # abstract
         pass
