@@ -14,7 +14,6 @@ Run using TestNet with JSON-RPC API at port 10332 and REST API at port 8080:
 
 See also:
 
-* If you encounter any issues, please report them here: https://github.com/CityOfZion/neo-python/issues/273
 * Server setup
   * Guide for Ubuntu server setup: https://gist.github.com/metachris/2be27cdff9503ebe7db1c27bfc60e435
   * Systemd service config: https://gist.github.com/metachris/03d1cc47df7cddfbc4009d5249bdfc6c
@@ -25,42 +24,30 @@ Logging
 
 This api-server can log to stdout/stderr, logfile and syslog.
 Check `api-server.py -h` for more details.
-
-Twisted uses a quite custom logging setup. Here we simply setup the Twisted logger
-to reuse our logzero logging setup. See also:
-
-* http://twisted.readthedocs.io/en/twisted-17.9.0/core/howto/logger.html
-* https://twistedmatrix.com/documents/17.9.0/api/twisted.logger.STDLibLogObserver.html
 """
+import argparse
+import asyncio
 import os
 import sys
-import argparse
-import threading
-from time import sleep
 from logging.handlers import SysLogHandler
 
 import logzero
 from logzero import logger
-from prompt_toolkit import prompt
-
-# Twisted logging
-from twisted.logger import STDLibLogObserver, globalLogPublisher
-
-# Twisted and Klein methods and modules
-from twisted.internet import reactor, task, endpoints, threads
-from twisted.web.server import Site
+from neo.Network.common import blocking_prompt as prompt
+from aiohttp import web
+from signal import SIGINT
 
 # neo methods and modules
 from neo.Core.Blockchain import Blockchain
-from neo.Implementations.Blockchains.LevelDB.LevelDBBlockchain import LevelDBBlockchain
-from neo.Implementations.Notifications.LevelDB.NotificationDB import NotificationDB
+from neo.Storage.Implementation.DBFactory import getBlockchainDB
+from neo.Implementations.Notifications.NotificationDB import NotificationDB
 from neo.Wallets.utils import to_aes_key
 from neo.Implementations.Wallets.peewee.UserWallet import UserWallet
-
-from neo.Network.NodeLeader import NodeLeader
+from neo.Network.p2pservice import NetworkService
 from neo.Settings import settings
 from neo.Utils.plugin import load_class_from_path
-import neo.Settings
+from neo.Wallets.utils import to_aes_key
+from contextlib import suppress
 
 # Logfile default settings (only used if --logfile arg is used)
 LOGFILE_MAX_BYTES = 5e7  # 50 MB
@@ -79,7 +66,7 @@ def write_pid_file():
         f.write(str(os.getpid()))
 
 
-def custom_background_code():
+async def custom_background_code():
     """ Custom code run in a background thread.
 
     This function is run in a daemonized thread, which means it can be instantly killed at any
@@ -87,35 +74,11 @@ def custom_background_code():
     thread and handle exiting this thread in another way (eg. with signals and events).
     """
     while True:
-        logger.info("[%s] Block %s / %s", settings.net_name, str(Blockchain.Default().Height + 1), str(Blockchain.Default().HeaderHeight + 1))
-        sleep(15)
+        logger.info("[%s] Block %s / %s", settings.net_name, str(Blockchain.Default().Height), str(Blockchain.Default().HeaderHeight))
+        await asyncio.sleep(15)
 
 
-def on_persistblocks_error(err):
-    logger.debug("On Persist blocks loop error! %s " % err)
-
-
-def stop_block_persisting():
-    global continue_persisting
-    continue_persisting = False
-
-
-def persist_done(value):
-    """persist callback. Value is unused"""
-    if continue_persisting:
-        start_block_persisting()
-    else:
-        block_deferred.cancel()
-
-
-def start_block_persisting():
-    global block_deferred
-    block_deferred = threads.deferToThread(Blockchain.Default().PersistBlocks)
-    block_deferred.addCallback(persist_done)
-    block_deferred.addErrback(on_persistblocks_error)
-
-
-def main():
+async def setup_and_start(loop):
     parser = argparse.ArgumentParser()
 
     # Network options
@@ -136,7 +99,7 @@ def main():
     group_logging = parser.add_argument_group(title="Logging options")
     group_logging.add_argument("--logfile", action="store", type=str, help="Logfile")
     group_logging.add_argument("--syslog", action="store_true", help="Log to syslog instead of to log file ('user' is the default facility)")
-    group_logging.add_argument("--syslog-local", action="store", type=int, choices=range(0, 7), metavar="[0-7]",
+    group_logging.add_argument("--syslog-local", action="store", type=int, choices=range(0, 7 + 1), metavar="[0-7]",
                                help="Log to a local syslog facility instead of 'user'. Value must be between 0 and 7 (e.g. 0 for 'local0').")
     group_logging.add_argument("--disable-stderr", action="store_true", help="Disable stderr logger")
 
@@ -144,7 +107,10 @@ def main():
     parser.add_argument("--datadir", action="store",
                         help="Absolute path to use for database directories")
     # peers
-    parser.add_argument("--maxpeers", action="store", default=5,
+    parser.add_argument("--minpeers", action="store", type=int, choices=range(1, 10 + 1), metavar="[1-10]",
+                        help="Min peers to use for P2P Joining")
+
+    parser.add_argument("--maxpeers", action="store", type=int, choices=range(1, 10 + 1), metavar="[1-10]",
                         help="Max peers to use for P2P Joining")
 
     # If a wallet should be opened
@@ -162,17 +128,17 @@ def main():
     if not args.port_rpc and not args.port_rest:
         print("Error: specify at least one of --port-rpc / --port-rest")
         parser.print_help()
-        return
+        raise SystemExit
 
     if args.port_rpc == args.port_rest:
         print("Error: --port-rpc and --port-rest cannot be the same")
         parser.print_help()
-        return
+        raise SystemExit
 
     if args.logfile and (args.syslog or args.syslog_local):
         print("Error: Cannot only use logfile or syslog at once")
         parser.print_help()
-        return
+        raise SystemExit
 
     # Setting the datadir must come before setting the network, else the wrong path is checked at net setup.
     if args.datadir:
@@ -190,13 +156,45 @@ def main():
     elif args.coznet:
         settings.setup_coznet()
 
-    if args.maxpeers:
+    def set_min_peers(num_peers) -> bool:
         try:
-            settings.set_max_peers(args.maxpeers)
-            print("Maxpeers set to ", args.maxpeers)
+            settings.set_min_peers(num_peers)
+            print("Minpeers set to ", num_peers)
+            return True
+        except ValueError:
+            print("Please supply a positive integer for minpeers")
+            return False
+
+    def set_max_peers(num_peers) -> bool:
+        try:
+            settings.set_max_peers(num_peers)
+            print("Maxpeers set to ", num_peers)
+            return True
         except ValueError:
             print("Please supply a positive integer for maxpeers")
-            return  
+            return False
+
+    minpeers = args.minpeers
+    maxpeers = args.maxpeers
+
+    if minpeers and maxpeers:
+        if minpeers > maxpeers:
+            print("minpeers setting cannot be bigger than maxpeers setting")
+            return
+        if not set_min_peers(minpeers) or not set_max_peers(maxpeers):
+            return
+    elif minpeers:
+        if not set_min_peers(minpeers):
+            return
+        if minpeers > settings.CONNECTED_PEER_MAX:
+            if not set_max_peers(minpeers):
+                return
+    elif maxpeers:
+        if not set_max_peers(maxpeers):
+            return
+        if maxpeers < settings.CONNECTED_PEER_MIN:
+            if not set_min_peers(maxpeers):
+                return
 
     if args.syslog or args.syslog_local is not None:
         # Setup the syslog facility
@@ -229,11 +227,16 @@ def main():
 
         passwd = os.environ.get('NEO_PYTHON_JSONRPC_WALLET_PASSWORD', None)
         if not passwd:
-            passwd = prompt("[password]> ", is_password=True)
+            try:
+                passwd = prompt("[password]> ", is_password=True)
+            except KeyboardInterrupt:
+                print("Wallet opening cancelled")
+                return
 
         password_key = to_aes_key(passwd)
         try:
             wallet = UserWallet.Open(args.wallet, password_key)
+            asyncio.create_task(wallet.sync_wallet(start_block=wallet._current_height))
 
         except Exception as e:
             print(f"Could not open wallet {e}")
@@ -247,35 +250,16 @@ def main():
     # Write a PID file to easily quit the service
     write_pid_file()
 
-    # Setup Twisted and Klein logging to use the logzero setup
-    observer = STDLibLogObserver(name=logzero.LOGZERO_DEFAULT_LOGGER)
-    globalLogPublisher.addObserver(observer)
-
-    def loopingCallErrorHandler(error):
-        logger.info("Error in loop: %s " % error)
-
     # Instantiate the blockchain and subscribe to notifications
-    blockchain = LevelDBBlockchain(settings.chain_leveldb_path)
+    blockchain = Blockchain(getBlockchainDB())
     Blockchain.RegisterBlockchain(blockchain)
 
-    start_block_persisting()
+    p2p = NetworkService()
+    p2p_task = loop.create_task(p2p.start())
+    loop.create_task(custom_background_code())
 
-    # If a wallet is open, make sure it processes blocks
-    if wallet:
-        walletdb_loop = task.LoopingCall(wallet.ProcessBlocks)
-        wallet_loop_deferred = walletdb_loop.start(1)
-        wallet_loop_deferred.addErrback(loopingCallErrorHandler)
-
-    # Setup twisted reactor, NodeLeader and start the NotificationDB
-    reactor.suggestThreadPoolSize(15)
-    NodeLeader.Instance().Start()
     NotificationDB.instance().start()
 
-    # Start a thread with custom code
-    d = threading.Thread(target=custom_background_code)
-    d.setDaemon(True)  # daemonizing the thread will kill it when the main thread is quit
-    d.start()
-
     if args.port_rpc:
         logger.info("Starting json-rpc api server on http://%s:%s" % (args.host, args.port_rpc))
         try:
@@ -283,10 +267,12 @@ def main():
         except ValueError as err:
             logger.error(err)
             sys.exit()
-        api_server_rpc = rpc_class(args.port_rpc, wallet=wallet)
+        api_server_rpc = rpc_class(wallet=wallet)
 
-        endpoint_rpc = "tcp:port={0}:interface={1}".format(args.port_rpc, args.host)
-        endpoints.serverFromString(reactor, endpoint_rpc).listen(Site(api_server_rpc.app.resource()))
+        runner = web.AppRunner(api_server_rpc.app)
+        await runner.setup()
+        site = web.TCPSite(runner, args.host, args.port_rpc)
+        await site.start()
 
     if args.port_rest:
         logger.info("Starting REST api server on http://%s:%s" % (args.host, args.port_rest))
@@ -296,17 +282,50 @@ def main():
             logger.error(err)
             sys.exit()
         api_server_rest = rest_api()
-        endpoint_rest = "tcp:port={0}:interface={1}".format(args.port_rest, args.host)
-        endpoints.serverFromString(reactor, endpoint_rest).listen(Site(api_server_rest.app.resource()))
+        runner = web.AppRunner(api_server_rest.app)
+        await runner.setup()
+        site = web.TCPSite(runner, args.host, args.port_rpc)
+        await site.start()
 
-    reactor.addSystemEventTrigger('before', 'shutdown', stop_block_persisting)
-    reactor.run()
+    return wallet
+
+
+async def shutdown():
+    # cleanup any remaining tasks
+    all_tasks = asyncio.all_tasks()
+    for task in all_tasks:
+        task.cancel()
+        with suppress((asyncio.CancelledError, Exception)):
+            await task
+
+
+def system_exit():
+    raise SystemExit
+
+
+def main():
+    loop = asyncio.get_event_loop()
+
+    # because a KeyboardInterrupt is so violent it can shutdown the DB in an unpredictable state.
+    loop.add_signal_handler(SIGINT, system_exit)
+    main_task = loop.create_task(setup_and_start(loop))
+
+    try:
+        loop.run_forever()
+    except SystemExit:
+        p2p = NetworkService()
+        loop.run_until_complete(p2p.shutdown())
+        loop.run_until_complete(shutdown())
+        loop.run_until_complete(loop.shutdown_asyncgens())
+        loop.stop()
+    finally:
+        loop.close()
 
-    # After the reactor is stopped, gracefully shutdown the database.
     logger.info("Closing databases...")
     NotificationDB.close()
     Blockchain.Default().Dispose()
-    NodeLeader.Instance().Shutdown()
+
+    wallet = main_task.result()
     if wallet:
         wallet.Close()
 
